//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Mask/MaskEditorPropertyPanel.cpp
//! @brief     Implements class MaskEditorPropertyPanel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Mask/MaskEditorPropertyPanel.h"
#include "GUI/Model/Data/IntensityDataItem.h"
#include "GUI/Model/Session/SessionModel.h"
#include "GUI/View/PropertyEditor/ComponentTreeView.h"
#include "GUI/View/Tool/GroupBoxCollapser.h"

#include <QGroupBox>
#include <QListView>
#include <QVBoxLayout>

MaskEditorPropertyPanel::MaskEditorPropertyPanel(QWidget* parent)
    : QWidget(parent)
    , m_listView(new QListView)
    , m_maskPropertyEditor(new ComponentTreeView)
    , m_plotPropertyEditor(new ComponentTreeView)
    , m_maskModel(nullptr)
    , m_intensityDataItem(nullptr)
{
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
    setObjectName(QLatin1String("MaskEditorToolPanel"));

    m_listView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(m_listView, &QListView::customContextMenuRequested, this,
            &MaskEditorPropertyPanel::onCustomContextMenuRequested);

    auto* mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(8);

    // -- plot properties
    auto* plotPropertiesGroup = new QGroupBox("Plot properties", this);
    auto* plotPropertiesLayout = new QVBoxLayout(plotPropertiesGroup);
    plotPropertiesLayout->setContentsMargins(0, 0, 0, 0);
    m_plotPropertyEditor->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
    plotPropertiesLayout->addWidget(m_plotPropertyEditor);
    GroupBoxCollapser::installIntoGroupBox(plotPropertiesGroup);

    // -- mask stack
    auto* maskStackGroup = new QGroupBox("Mask stack", this);
    auto* maskStackLayout = new QVBoxLayout(maskStackGroup);
    maskStackLayout->setContentsMargins(0, 0, 0, 0);
    m_listView->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
    maskStackLayout->addWidget(m_listView);
    GroupBoxCollapser::installIntoGroupBox(maskStackGroup);

    // -- mask properties
    auto* maskPropertiesGroup = new QGroupBox("Mask properties", this);
    auto* maskPropertiesLayout = new QVBoxLayout(maskPropertiesGroup);
    maskPropertiesLayout->setContentsMargins(0, 0, 0, 0);
    m_maskPropertyEditor->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
    maskPropertiesLayout->addWidget(m_maskPropertyEditor);
    GroupBoxCollapser::installIntoGroupBox(maskPropertiesGroup);

    mainLayout->addWidget(plotPropertiesGroup);
    mainLayout->addWidget(maskStackGroup);
    mainLayout->addWidget(maskPropertiesGroup);
    mainLayout->addSpacerItem(
        new QSpacerItem(0, 10, QSizePolicy::Fixed, QSizePolicy::MinimumExpanding));

    setLayout(mainLayout);

    m_plotPropertyEditor->setShowHeader(false);
}

QSize MaskEditorPropertyPanel::sizeHint() const
{
    return QSize(128, 128);
}

QSize MaskEditorPropertyPanel::minimumSizeHint() const
{
    return QSize(128, 128);
}

void MaskEditorPropertyPanel::setMaskContext(SessionModel* model,
                                             const QModelIndex& maskContainerIndex,
                                             IntensityDataItem* intensityItem)
{
    m_maskModel = model;
    m_rootIndex = maskContainerIndex;
    m_intensityDataItem = intensityItem;

    m_listView->setModel(m_maskModel);
    m_listView->setSelectionMode(QAbstractItemView::ExtendedSelection);

    m_listView->setRootIndex(m_rootIndex);

    connect(m_listView->selectionModel(), &QItemSelectionModel::selectionChanged, this,
            &MaskEditorPropertyPanel::onSelectionChanged, Qt::UniqueConnection);

    m_plotPropertyEditor->setItem(m_intensityDataItem);
}

void MaskEditorPropertyPanel::resetContext()
{
    m_maskModel = nullptr;
    m_rootIndex = {};
    m_intensityDataItem = nullptr;
    m_listView->setModel(nullptr);
    m_maskPropertyEditor->setItem(nullptr);
    m_plotPropertyEditor->setItem(nullptr);
}

QItemSelectionModel* MaskEditorPropertyPanel::selectionModel()
{
    ASSERT(m_listView);
    return m_listView->selectionModel();
}

//! Show/Hide panel. When panel is hidden, all property editors are disabled.
void MaskEditorPropertyPanel::setPanelHidden(bool value)
{
    this->setHidden(value);

    if (!m_rootIndex.isValid())
        return;

    if (value) {
        m_maskPropertyEditor->setItem(nullptr);
        m_plotPropertyEditor->setItem(nullptr);
    } else {
        QModelIndexList indexes = selectionModel()->selectedIndexes();
        if (!indexes.empty())
            m_maskPropertyEditor->setItem(m_maskModel->itemForIndex(indexes.front()));

        m_plotPropertyEditor->setItem(m_intensityDataItem);
    }
}

void MaskEditorPropertyPanel::onSelectionChanged(const QItemSelection& selected,
                                                 const QItemSelection&)
{
    if (!selected.empty())
        m_maskPropertyEditor->setItem(m_maskModel->itemForIndex(selected.indexes().front()));
    else
        m_maskPropertyEditor->setItem(nullptr);
}

void MaskEditorPropertyPanel::onCustomContextMenuRequested(const QPoint& point)
{
    emit itemContextMenuRequest(m_listView->mapToGlobal(point));
}
