//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/PropertyEditor/ComponentTreeView.cpp
//! @brief     Implements class ComponentTreeView
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/PropertyEditor/ComponentTreeView.h"
#include "GUI/Model/Component/ComponentProxyModel.h"
#include "GUI/Model/Session/SessionModel.h"
#include "GUI/View/PropertyEditor/CustomEventFilters.h"
#include "GUI/View/PropertyEditor/SessionModelDelegate.h"
#include "GUI/View/Tool/StyleUtils.h"
#include <QAction>
#include <QBoxLayout>
#include <QMenu>
#include <QStandardItemModel>
#include <QTreeView>

ComponentTreeView::ComponentTreeView(QWidget* parent)
    : QWidget(parent)
    , m_tree(new QTreeView)
    , m_delegate(new SessionModelDelegate(this))
    , m_proxyModel(new ComponentProxyModel(this))
    , m_placeHolderModel(new QStandardItemModel(this))
    , m_eventFilter(new RightMouseButtonEater)
{
    auto* layout = new QVBoxLayout;

    layout->setMargin(0);
    layout->setSpacing(0);
    layout->addWidget(m_tree);

    setLayout(layout);

    QStringList labels = {"Name", "Value"};
    m_placeHolderModel->setHorizontalHeaderLabels(labels);

    GUI::Util::Style::setPropertyStyle(m_tree);
    m_tree->setRootIsDecorated(false);
    m_tree->setModel(m_placeHolderModel);
    m_tree->setItemDelegate(m_delegate);

    // provide one click editing, but still keeping custom context menu alive
    m_tree->setEditTriggers(QAbstractItemView::AllEditTriggers);
    m_tree->viewport()->installEventFilter(m_eventFilter.get());

    // custom context menu setup
    m_tree->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(m_tree, &QTreeView::customContextMenuRequested, this,
            &ComponentTreeView::onCustomContextMenuRequested);

    setShowHeader(true);
}

void ComponentTreeView::setItem(SessionItem* item)
{
    if (!item) {
        setModel(nullptr);
        return;
    }
    setModel(item->model());
    setRootIndex(item->index());
    m_tree->expandAll();
}

void ComponentTreeView::clearEditor()
{
    m_tree->setModel(m_placeHolderModel);
}

void ComponentTreeView::setModel(SessionModel* model)
{
    m_proxyModel->setSessionModel(model);
    if (model)
        m_tree->setModel(m_proxyModel);
    else
        m_tree->setModel(m_placeHolderModel);
}

void ComponentTreeView::setRootIndex(const QModelIndex& index)
{
    if (QWidget* editor = m_tree->indexWidget(m_tree->currentIndex()))
        m_delegate->closeEditor(editor, QAbstractItemDelegate::NoHint);
    ASSERT(m_proxyModel);
    m_proxyModel->setRootIndex(index);
    m_tree->setRootIndex(m_proxyModel->mapFromSource(index));
}

void ComponentTreeView::setShowHeader(bool show)
{
    m_tree->setHeaderHidden(!show);
}

void ComponentTreeView::onCustomContextMenuRequested(const QPoint& pos)
{
    auto point = m_tree->mapToGlobal(pos);
    auto treeIndex = m_tree->indexAt(pos);
    if (!treeIndex.isValid())
        return;

    auto index = m_proxyModel->mapToSource(treeIndex);

    auto* item = static_cast<SessionItem*>(index.internalPointer());
    if (item->value().type() != QVariant::Double)
        return;

    const bool sc_editor = item->editorType() == "ScientificDouble";

    // Creates right-mouse-click context menu on top of ComponentTreeView
    // which will allow user to switch between scientific notation and the notation
    // with a specified number of decimals.
    QMenu menu;
    QAction* scientificAction = menu.addAction("Scientific presentation");
    scientificAction->setCheckable(true);
    auto* doubleMenu = menu.addMenu("Double presentation");

    // To select scientific notation
    scientificAction->setChecked(sc_editor);
    connect(scientificAction, &QAction::triggered, [&]() {
        if (scientificAction->isChecked())
            item->setEditorType("ScientificDouble");
        else
            item->setEditorType("Default");
    });

    // to select number of decimals
    const int nmaxdigits = 8;
    for (int i = 1; i <= nmaxdigits; ++i) {
        auto* action = doubleMenu->addAction(QString("%1 digits").arg(i));
        if (!sc_editor && item->decimals() == i)
            action->setChecked(true);
        connect(action, &QAction::triggered, [i, &item] {
            item->setEditorType("Default");
            item->setDecimals(i);
        });
    }
    menu.exec(point);
}
